@@ -13,20 +13,20 @@ pub fn get_config() -> (Config, usize) {
         .arg(Arg::with_name("url")
             .short("u")
             .long("url")
-            .help("You can add a custom injection point with %s")
+            .help("You can add a custom injection point with %s.")
             .takes_value(true)
             .conflicts_with("request")
         )
         .arg(Arg::with_name("request")
             .short("r")
             .long("request")
-            .help("The file with raw http request")
+            .help("The file with the raw http request")
             .takes_value(true)
             .conflicts_with("url")
         )
         .arg(Arg::with_name("proto")
             .long("proto")
-            .help("Uses when the request file is present. (default is \"https\")")
+            .help("Protocol to use with request file (default is \"https\")")
             .takes_value(true)
             .requires("request")
             .conflicts_with("url")
@@ -99,7 +99,6 @@ pub fn get_config() -> (Config, usize) {
         .arg(
             Arg::with_name("headers")
                 .short("H")
-                .long("header")
                 .help("Example: -H 'one:one' 'two:two'")
                 .takes_value(true)
                 .min_values(1)
@@ -109,6 +108,13 @@ pub fn get_config() -> (Config, usize) {
                 .long("as-body")
                 .help("Send parameters via body.\nBuilt in body types that can be detected automatically: json, urlencode")
         )
+        .arg(
+            Arg::with_name("headers-discovery")
+                .long("headers")
+                .help("Switch to header discovery mode.\nForbidden chars would be automatically removed from headers names")
+                .conflicts_with("as-body")
+                .conflicts_with("param-template")
+        )
         .arg(
             Arg::with_name("force")
                 .long("force")
@@ -136,19 +142,19 @@ pub fn get_config() -> (Config, usize) {
         .arg(
             Arg::with_name("keep-newlines")
                 .long("keep-newlines")
-                .help("--body 'a\\r\\nb' -> --body 'a{{new_line}}b'.\nWorks with body and parameter templates only")
+                .help("--body 'a\\r\\nb' -> --body 'a{{new_line}}b'.\nWorks with body and parameter templates only.")
             )
         .arg(
             Arg::with_name("replay-once")
                 .long("replay-once")
-                .help("If replay proxy is specified, send all found parameters within one request")
+                .help("If replay proxy is specified, send all found parameters within one request.")
                 .requires("replay-proxy")
         )
         .arg(
             Arg::with_name("replay-proxy")
                 .takes_value(true)
                 .long("replay-proxy")
-                .help("Request target with every found parameter via replay proxy at the end")
+                .help("Request target with every found parameter via replay proxy at the end.")
         )
         .arg(
             Arg::with_name("custom-parameters")
@@ -220,7 +226,7 @@ pub fn get_config() -> (Config, usize) {
             Arg::with_name("max")
                 .short("m")
                 .long("max")
-                .help("Change the maximum number of parameters. (default is 128/192/256 for query and 512 for body)")
+                .help("Change the maximum number of parameters.\n(default is 128/192/256 for query, 64/128/196 for headers and 512 for body)")
                 .takes_value(true)
         )
         .arg(
@@ -234,6 +240,11 @@ pub fn get_config() -> (Config, usize) {
                 .long("http2")
                 .help("Prefer http/2 over http/1.1")
                 .conflicts_with("request")
+        )
+        .arg(
+            Arg::with_name("verify")
+                .long("verify")
+                .help("Verify found parameters one more time.")
         );
 
     let args = app.clone().get_matches();
@@ -260,8 +271,10 @@ pub fn get_config() -> (Config, usize) {
         None => {
             if args.is_present("as-body") {
                 512
-            } else {
+            } else if !args.is_present("headers-discovery") {
                 128
+            } else {
+                64
             }
         }
     };
@@ -306,6 +319,7 @@ pub fn get_config() -> (Config, usize) {
     };
 
     let mut headers: HashMap<String, String> = HashMap::new();
+    let mut within_headers: bool = false;
     if let Some(val) = args.values_of("headers") {
         for header in val {
             let mut k_v = header.split(':');
@@ -327,6 +341,10 @@ pub fn get_config() -> (Config, usize) {
                 k_v.map(|x| ":".to_owned() + x).collect(),
             ].concat();
 
+            if value.contains("%s") {
+                within_headers = true;
+            }
+
             headers.insert(key.to_string(), value);
         }
     };
@@ -416,12 +434,22 @@ pub fn get_config() -> (Config, usize) {
         .unwrap_or("https://something.something")
         .to_string();
 
-    if !args.is_present("as-body") && url.contains('?') && url.contains('=') && !url.contains("%s") {
-        url.push_str("&%s");
-        path.push_str("&%s");
-    } else if !args.is_present("as-body") && !url.contains("%s") {
-        url.push_str("?%s");
-        path.push_str("?%s");
+    if !args.is_present("as-body") && !within_headers && !args.is_present("headers-discovery") && url.contains('?') && url.contains('=') && !url.contains("%s") {
+        if args.is_present("encode") {
+            url.push_str("%26%s");
+            path.push_str("%26%s");
+        } else {
+            url.push_str("&%s");
+            path.push_str("&%s");
+        }
+    } else if !args.is_present("as-body") && !within_headers &&!args.is_present("headers-discovery") && !url.contains("%s") {
+        if args.is_present("encode") {
+            url.push_str("%3f%s");
+            path.push_str("%3f%s");
+        } else {
+            url.push_str("?%s");
+            path.push_str("?%s");
+        }
     }
 
     let parameter_template = match args.is_present("keep-newlines") {
@@ -440,12 +468,13 @@ pub fn get_config() -> (Config, usize) {
     if parameter_template.is_empty() {
         if body_type.contains("json") && args.is_present("as-body") {
             parameter_template = "\"%k\":\"%v\", ";
+        } else if within_headers {
+            parameter_template = "%k=%v; ";
         } else {
             parameter_template = "%k=%v&";
         }
     }
 
-
     let custom_keys: Vec<String> = match args.values_of("custom-parameters") {
         Some(val) => {
             val.map(|x| x.to_string()).collect()
@@ -510,6 +539,8 @@ pub fn get_config() -> (Config, usize) {
         save_responses: args.value_of("save-responses").unwrap_or("").to_string(),
         output_format: args.value_of("output-format").unwrap_or("").to_string(),
         as_body: args.is_present("as-body"),
+        headers_discovery: args.is_present("headers-discovery"),
+        within_headers,
         force: args.is_present("force"),
         disable_response_correction: args.is_present("disable-response-correction"),
         disable_custom_parameters: args.is_present("disable-custom-parameters"),
@@ -525,7 +556,8 @@ pub fn get_config() -> (Config, usize) {
         learn_requests_count,
         max,
         concurrency,
-        http2: args.is_present("http2")
+        http2: args.is_present("http2"),
+        verify: args.is_present("verify")
     };
 
     config = if !request.is_empty() {
