@@ -108,36 +108,45 @@ pub fn heuristic(body: &str) -> Vec<String> {
     found
 }
 
-pub fn generate_request(config: &Config, initial_query: &HashMap<String, String>) -> String {
-    let mut query: HashMap<String, String> = HashMap::with_capacity(initial_query.len());
-    for (k, v) in initial_query.iter() {
-        query.insert(k.to_string(), v.replace("%random%_", ""));
+//remove forbidden characters from header name, otherwise reqwest throws errors
+pub fn fix_headers<'a>(header: &'a str) -> Option<String> {
+    lazy_static! {
+        static ref RE: Regex = Regex::new(r"[^!-'*+\-\.0-9a-zA-Z^-`|~]").unwrap();
     }
 
-    let mut req: String = String::with_capacity(1024);
+    if RE.is_match(header) {
+        Some(RE.replace_all(header, "").to_string())
+    } else {
+        None
+    }
+}
+
+pub fn generate_request(config: &Config, initial_query: &HashMap<String, String>) -> String {
+    let mut hashmap_query: HashMap<String, String> = HashMap::with_capacity(initial_query.len());
+    for (k, v) in initial_query.iter() {
+        hashmap_query.insert(k.to_string(), v.replace("%random%_", ""));
+    }
+
+    let query: String = if !hashmap_query.is_empty() {
+        if config.as_body {
+            make_body(&config, &hashmap_query)
+        } else if config.within_headers {
+            make_header_value(&config, &hashmap_query)
+        } else if config.headers_discovery {
+            String::new()
+        } else {
+            make_query(&config, &hashmap_query)
+        }
+    } else {
+        String::new()
+    };
+
+    let mut req: String = String::with_capacity(4096);
     req.push_str(&config.url);
     req.push('\n');
     req.push_str(&config.method);
     req.push(' ');
-
-    if !config.as_body {
-        let mut query_string = String::new();
-        for (k, v) in query.iter() {
-            query_string.push_str(k);
-            query_string.push('=');
-            query_string.push_str(v);
-            query_string.push('&');
-        }
-        query_string.pop(); //remove the last &
-
-        query_string = if config.encode {
-            utf8_percent_encode(&query_string, &FRAGMENT).to_string()
-        } else {
-            query_string
-        };
-
-        req.push_str(&config.path.replace("%s", &query_string));
-    }
+    req.push_str(&config.path.replace("%s", &query));
 
     if config.http2 {
         req.push_str(" HTTP/2\n");
@@ -153,19 +162,26 @@ pub fn generate_request(config: &Config, initial_query: &HashMap<String, String>
     for (key, value) in config.headers.iter() {
         req.push_str(key);
         req.push_str(": ");
-        req.push_str(&value.replace("{{random}}", &random_line(config.value_size)));
+        if value.contains("%s") && config.headers_discovery && config.within_headers {
+            req.push_str(&value.replace("%s", &query).replace("{{random}}", &random_line(config.value_size)));
+        } else {
+            req.push_str(&value.replace("{{random}}", &random_line(config.value_size)));
+        }
         req.push('\n');
     }
 
-    let body: String = if config.as_body && !query.is_empty() {
-        make_body(&config, &query)
-    } else {
-        config.body.to_owned()
-    };
+    if config.headers_discovery && !config.within_headers {
+        for (key, value) in hashmap_query.iter() {
+            req.push_str(key);
+            req.push_str(": ");
+            req.push_str(&value.replace("{{random}}", &random_line(config.value_size)));
+            req.push('\n');
+        }
+    }
 
-    if !body.is_empty() {
+    if config.as_body && !query.is_empty() {
         req.push('\n');
-        req.push_str(&body);
+        req.push_str(&query);
         req.push('\n');
     }
 
@@ -215,6 +231,10 @@ pub fn adjust_body(body: &str, t: &str) -> String {
     }
 }
 
+pub fn make_header_value(config: &Config, query: &HashMap<String, String>) -> String {
+    make_query(config, query)
+}
+
 pub fn make_body(config: &Config, query: &HashMap<String, String>) -> String {
     let mut body: String = String::new();
 
@@ -238,8 +258,8 @@ pub fn make_body(config: &Config, query: &HashMap<String, String>) -> String {
     body
 }
 
-pub fn make_query(params: &HashMap<String, String>, config: &Config) -> String {
-    let mut query: String = String::from("");
+pub fn make_query(config: &Config, params: &HashMap<String, String>) -> String {
+    let mut query: String = String::new();
 
     for (k, v) in params {
         query = query + &config.parameter_template.replace("%k", k).replace("%v", v);
@@ -275,26 +295,18 @@ pub fn make_hashmap(
     hashmap
 }
 
-pub fn parse_request(config: Config, proto: &str, request: &str, custom_value_template: bool) -> Option<Config> {
+pub fn parse_request(config: Config, proto: &str, request: &str, custom_parameter_template: bool) -> Option<Config> {
     let mut lines = request.lines();
     let mut host = String::new();
     let mut content_type = String::new();
     let mut headers: HashMap<String, String> = config.headers.clone();
+    let mut within_headers: bool = config.within_headers;
     let mut firstline = lines.next()?.split(' ');
     let method = firstline.next()?.to_string();
     let mut path = firstline.next()?.to_string();
 
     let http2: bool = firstline.next()?.to_string().contains("HTTP/2");
 
-    let mut parameter_template = if !custom_value_template {
-        match config.body_type == "json" {
-            true => String::from("\"%k\":\"%v\", "),
-            false => String::from("%k=%v&")
-        }
-    } else {
-        config.parameter_template.clone()
-    };
-
     //read headers
     while let Some(line) = lines.next() {
         if line.is_empty() {
@@ -308,6 +320,10 @@ pub fn parse_request(config: Config, proto: &str, request: &str, custom_value_te
             k_v.map(|x| ":".to_owned() + x).collect(),
         ].concat();
 
+        if value.contains("%s") {
+            within_headers = true;
+        }
+
         match key.to_lowercase().as_str() {
             "content-type" => content_type = value.clone(),
             "host" => {
@@ -323,6 +339,19 @@ pub fn parse_request(config: Config, proto: &str, request: &str, custom_value_te
         headers.insert(key.to_string(), value);
     }
 
+    let mut parameter_template = if !custom_parameter_template {
+        if config.within_headers {
+            String::from("%k=%v; ")
+        } else {
+            match config.body_type == "json" {
+                true => String::from("\"%k\":\"%v\", "),
+                false => String::from("%k=%v&")
+            }
+        }
+    } else {
+        config.parameter_template.clone()
+    };
+
     let mut body = lines.next().unwrap_or("").to_string();
     while let Some(part) = lines.next() {
         if !part.is_empty() {
@@ -332,7 +361,7 @@ pub fn parse_request(config: Config, proto: &str, request: &str, custom_value_te
     }
 
     //check whether the body type can be json
-    let body_type = if config.body_type.contains('-') && config.as_body && !custom_value_template
+    let body_type = if config.body_type.contains('-') && config.as_body && !custom_parameter_template
     && (
         content_type.contains("json") || (!body.is_empty() && body.starts_with('{') )
     ) {
@@ -351,10 +380,11 @@ pub fn parse_request(config: Config, proto: &str, request: &str, custom_value_te
 
     let mut url = [proto,"://", &host, &path].concat();
     let initial_url = url.clone();
-    if !config.as_body && url.contains('?') && url.contains('=') && !url.contains("%s") {
+
+    if !config.as_body && url.contains('?') && !within_headers && !config.headers_discovery && url.contains('=') && !url.contains("%s") {
         url.push_str("&%s");
         path.push_str("&%s");
-    } else if !config.as_body {
+    } else if !config.as_body && !within_headers && !config.headers_discovery {
         url.push_str("?%s");
         path.push_str("?%s");
     }
@@ -365,6 +395,7 @@ pub fn parse_request(config: Config, proto: &str, request: &str, custom_value_te
         host,
         path,
         headers,
+        within_headers,
         body,
         body_type,
         parameter_template,
@@ -392,20 +423,29 @@ where
 pub fn create_output(config: &Config, found_params: Vec<String>) -> String {
     match config.output_format.as_str() {
         "url" => {
-            let mut line = match config.initial_url.contains('?') {
-                true => config.initial_url.to_owned()+"&",
-                false => config.initial_url.to_owned()+"?"
+            let mut line = if !found_params.is_empty() {
+                match config.initial_url.contains('?') {
+                    true => config.initial_url.to_owned()+"&",
+                    false => config.initial_url.to_owned()+"?"
+                }
+            } else {
+                config.initial_url.clone()
             };
 
-            for param in &found_params {
-                line.push_str(&param);
-                if !param.contains('=') {
-                    line.push('=');
-                    line.push_str(&random_line(config.value_size));
+            if !found_params.is_empty() {
+
+                for param in &found_params {
+                    line.push_str(&param);
+                    if !param.contains('=') {
+                        line.push('=');
+                        line.push_str(&random_line(config.value_size));
+                    }
+                    line.push('&')
                 }
-                line.push('&')
+
+                line.pop();
             }
-            line.pop();
+
             line.push('\n');
 
             line
@@ -417,26 +457,35 @@ pub fn create_output(config: &Config, found_params: Vec<String>) -> String {
                 &config.initial_url
             );
 
-            for param in &found_params {
-                line.push('\"');
-                line.push_str(&param.replace("\"", "\\\""));
-                line.push('\"');
-                line.push_str(", ");
+            if !found_params.is_empty() {
+
+                for param in &found_params {
+                    line.push('\"');
+                    line.push_str(&param.replace("\"", "\\\""));
+                    line.push('\"');
+                    line.push_str(", ");
+                }
+
+                line = line[..line.len() - 2].to_string();
             }
 
-            let mut line = line[..line.len() - 2].to_string();
             line.push_str("]}\n");
+
             line
         },
         _ => {
             let mut line = format!("{} {} % ", &config.method, &config.initial_url);
 
-            for param in &found_params {
-                line.push_str(&param);
-                line.push_str(", ")
+            if !found_params.is_empty() {
+
+                for param in &found_params {
+                    line.push_str(&param);
+                    line.push_str(", ")
+                }
+
+                line = line[..line.len() - 2].to_string();
             }
 
-            line = line[..line.len() - 2].to_string();
             line.push('\n');
             line
         },
